#include <stdio.h>
#include <string.h>
#include "npclient.h"


static int hexchar2val(char c)
{
	if ((c >= '0') && (c <= '9')) {
		return c - '0';
	} else
	if ((c >= 'A') && (c <= 'F')) {
		return c - ('A' - 10);
	}
	else
	if ((c >= 'a') && (c <= 'f')) {
		return c - ('a' - 10);
	}
	return -1;
}


/*
** Get quoted string
** quote:
**	<0: use first source char as quote char
**	>=0 use this char as terminator
*/

int np_get_quoted_string(char *d,int dsz,int *ssz,char *s,int quote)
{
int j, k;
char *s0, *d0, c, qchar;

	s0 = s;
	if (quote < 0) {
		if (!(qchar = *s++))
			return -1;
	} else {
		qchar = quote;
	}

	d0 = d;
	for (;;) {
		if (!*s) {
			if (quote < 0) break;
			if (ssz) {
				*ssz = (s-s0);
			}
			*d = 0;
			return (int) (d-d0);
		}
		if ('\\' == *s) {
			if (!*++s) break;
			if ('n' == *s) {
				c = '\n';
			}
			else
			if ('r' == *s) {
				c = '\r';
			}
			else
			if ('t' == *s) {
				c = '\t';
			} else {
				c = 0;
			}
			if (c) {
				if (!dsz) {
					*d = 0;
					return -2;
				}
				*d++ = c;
				dsz --;
				s ++;
				continue;
			}
		} else if (qchar == *s) {
			if (quote < 0) s++;
			if (ssz) {
				*ssz = (s-s0);
			}
			*d = 0;
			return (int) (d-d0);
		} else if ('%' == *s) {	/* %XX	*/
			if ('%' != *++s) {
				k = hexchar2val(*s++);
				if (k < 0) break;
				j = hexchar2val(*s++);
				if (j < 0) break;
				j += 16 * k;
				if (!dsz) {
					*d = 0;
					return -2;
				}
				*d++ = (char) j;
				dsz --;
				continue;
			}
		}
		if (!dsz) {
			*d = 0;
			return -2;
		}
		*d++ = *s++;
		dsz --;
	}
	*d = 0;
	return -1;
}

/* Store quoted string from s to d prefixing with '\\'  */
/* return number of stored chars                        */
/* We support also: <\n>, <\r>, <\t>, %XX		*/

int np_store_quoted_string(char *d,char *s,int sz,
	int restricted,char quote)
{
char *t;

	/* Be sure that size(d) >= 3*size(s) because we have '%XX'	*/

	/* sz<0 => zero-terminated string	*/
	/* sz>=0 => sz bytes binary data	*/
	t = d;
	if (quote > 0) *t++ = quote;
	if ((sz > 0) || ((sz < 0) && *s))
	for (;;) {
		if ((*s == '\\') || (*s == quote && quote > 0) || (*s == '%')) {
			*t++ = '\\';
		} else if (*s == '\n') {
			*t++ = '\\';
			*t++ = 'n';
			s++;
			goto cont;
		} else if (*s == '\t') {
			*t++ = '\\';
			*t++ = 't';
			s++;
			goto cont;
		} else if (*s == '\r') {
			*t++ = '\\';
			*t++ = 'r';
			s++;
			goto cont;
		} else if (restricted && (
			(*s >= '0' && *s <= '9') ||
			(*s >= 'a' && *s <= 'z') ||
			(*s >= 'A' && *s <= 'Z'))) {
		} else if (restricted ||
			(*((unsigned char *) s) < 0x20) ||
			(*((unsigned char *) s) > 0x7E)) {
			*t ++ = '%';
			sprintf(t,"%02x",*((unsigned char *) s));
			t += 2;
			s ++;
			goto cont;
		}
		*t++ = *s++;

		cont:
		if (sz > 0) {
			if (! --sz) {
				break;
			}
		} else if (!*s) {
			break;
		}
	}
	if (quote > 0) *t++ = '"';
	*t = 0;
	return (int) (t-d);
}


